#include "Vista.h"
        SUBROUTINE FILTER(A,NROW,NCOL)

C       Routine to filter a Fourier domain image

C       This routine will add a gaussian or exponential disk to
C       an image.  The gaussian can have differing widths and
C       orientation.  If desired, the results can be computed
C       directly in the fourier domain.

C       Keywords:       FW=(yfw,xfw)            Enter y and x widths
C                       ANG=theta               PA of major axis

C       Author: Tod R. Lauer            Date: 7/29/82

        PARAMETER (PI=3.1415926, RADIAN=PI/180.0)
#ifdef VMS
        INCLUDE 'VINCLUDE:vistalink.inc'
        INCLUDE 'VINCLUDE:imagelink.inc'
#else
        INCLUDE 'vistadisk/source/include/vistalink.inc'
        INCLUDE 'vistadisk/source/include/imagelink.inc'
#endif
        COMPLEX A(NCOL/2,NROW)
        DIMENSION G(2)
        CHARACTER*8 PARM
        INTEGER ROW, COL
        LOGICAL SIG

C       Check command string

        IF (.NOT. GO) RETURN
        CALL KEYINIT
        CALL KEYDEF('FW=')
        CALL KEYDEF('ANG=')
        CALL KEYCHECK
        IF (XERR) RETURN

C       Initialize parameters

C       Default angle is 0
        THETA   =0.0
        SIG     =.FALSE.
        DO 2757 I=1, NCON
                IF (WORD(I)(1:3) .EQ. 'FW=') THEN
C                   Enter FWHM
                        CALL ASSIGNV(WORD(I),2,G,NG,PARM)
                        IF (XERR) RETURN
                        SIG     =.TRUE.
                        IF (NG .GE. 2) THEN
                                YWID    =G(1)
                                XWID    =G(2)
                        ELSE
                                YWID    =G(1)
                                XWID    =G(1)
                        END IF
                END IF

                IF (WORD(I)(1:4) .EQ. 'ANG=') THEN
C                   Enter theta
                        CALL ASSIGN(WORD(I),THETA,PARM)
                        IF (XERR) RETURN
                        THETA   =THETA*RADIAN
                END IF

2757        CONTINUE

        COSTH   =COS(THETA)
        SINTH   =SIN(THETA)
        IF (.NOT. SIG) THEN
50              PRINT 101
101             FORMAT (' Enter FWHM of gaussian in pixels: ',$)
                READ (*,*,ERR=50), F
                XWID    =F
                YWID    =F
        END IF

        OMY     =4.0*PI*YWID/FLOAT(NROW)
        OMX     =4.0*PI*XWID/FLOAT(NCOL-1)
        XWID    =XWID/2.0
        YWID    =YWID/2.0
        XSIG    =XWID*XWID/ALOG(2.0)
        YSIG    =YWID*YWID/ALOG(2.0)
        XSIG    =NROW*NROW/(PI*PI*XSIG)
        YSIG    =NCOL*NCOL/(PI*PI*YSIG)

C       Add gaussian to image

        XC      =1.0
        DO 2758 ROW=1,NROW
                IF (ROW .GT. NROW/2+1) THEN
                        YC      =FLOAT(NROW+1)
                ELSE
                        YC      =1.0
                END IF

                Y       =FLOAT(ROW)-YC
                YCS     =Y*COSTH
                YSN     =Y*SINTH
                IF (ROW .EQ. 1) THEN
                        SINCY   =1.0
                ELSE
                        SINCY   =SIN(OMY*Y)/(OMY*Y)
                END IF

                DO 2759 COL=1, NCOL/2
                        X       =FLOAT(COL)-XC
C                      New rotated x
                        XG      =X*COSTH+YSN
C                      New rotated y
                        YG      =YCS-X*SINTH
                        R2      =XG*XG/XSIG+YG*YG/YSIG
                        IF (COL .EQ. 1) THEN
                                SINCX   =1.0
                        ELSE
                                SINCX   =SIN(OMX*X)/(OMX*X)
                        END IF

                        IF (R2 .LT. 20.0) THEN
                                FILT    =EXP(-R2)-SINCY*SINCX
                        ELSE
                                FILT    =-SINCY*SINCX
                        END IF

                        A(COL,ROW)=A(COL,ROW)*FILT

2759            CONTINUE
2758   CONTINUE

        RETURN
        END
